/*
** Graph.c
** 
** Made by (Johnny Q. Hacker)
** Login   <solarion@johnathan>
** 

Copyright (C) 2008 Joseph Pingenot

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**Many thanks to Davyd Madeley for the excellent Cairo Widget Tutorial
**  at http://gnomejournal.org/article/34/writing-a-widget-using-cairo-and-gtk28

** Started on  Thu Jul 17 10:51:32 2008 Johnny Q. Hacker
** Last update Sun May 12 01:17:25 2002 Speed Blue
*/

#include "graphwidget.h"

G_DEFINE_TYPE(GraphWidget, graph_widget, GTK_TYPE_DRAWING_AREA);

struct drawing_context {
  GtkWidget *widget;
  cairo_t *cr;
  double radius;
};

static void draw_point(gpointer data, gpointer user_data) {
  struct drawing_context *cxt = (struct drawing_context *) user_data;
  cairo_arc(cxt->cr, ((*double)data)[0], ((*double)data)[1], cxt->radius, 0, 2*M_PI);
}

static void draw(GtkWidget *graph, cairo_t *cr) {
  struct drawing_context cxt;
  #ifdef DEBUG
    fprintf(stderr, "draw(%d, %d)\n", graph, cr);
    fprintf(stderr, "allocation.x=%d, allocation.y=%d, allocation.height=%d, allocation.width=%d\n", graph->allocation.x, graph->allocation.y, graph->allocation.height, graph->allocation.width);
  #endif
  double x0=graph->allocation.x;
  double y0=graph->allocation.y;
  double height=graph->allocation.height;
  double width=graph->allocation.width;
  cxt.widget = graph;
  cxt.cr = cr;
  cxt.radius = ((height < width)? height : width)*0.01;
  #ifdef DEBUG
  fprintf(stderr, "x0=%g, y0=%g, width=%g, height=%g\n", x0, y0, height, width);
  fprintf(stderr, "move_to(%d, %g->%g, %g->%g)\n", cr, graph_data[0], graph->allocation.x + graph_data[0]*(graph->allocation.width), graph_data[1], graph->allocation.y + graph_data[1]*(graph->allocation.height));
  #endif
  cairo_save(cr);
  cairo_translate(cr, x0, y0);
  cairo_scale(cr, width, height);
  g_ptr_array_foreach(GRAPH_WIDGET(graph)->graph->points, &draw_point, (gpointer)&cxt);
  cairo_set_source_rgb(cr, 0, 0, 0);
  cairo_stroke(cr);
  cairo_restore(cr);
}

static gboolean graph_widget_expose(GtkWidget *graph, GdkEventExpose *event) {
  cairo_t *cr = gdk_cairo_create(graph->window);
  cairo_rectangle(cr, event->area.x, event->area.y, event->area.width, event->area.height);
  cairo_clip(cr);
  draw(graph, cr);
  cairo_destroy(cr);
  return FALSE;
}

static void graph_widget_class_init(GraphWidgetClass *klass) {
  GtkWidgetClass *widget_class;
  widget_class = GTK_WIDGET_CLASS(klass);
  widget_class->expose_event = graph_widget_expose;
}

static void graph_widget_init(GraphWidget *graph) {
  /*Get a graph.*/
  graph->graph = graph_new();
}

GtkWidget *graph_widget_new() {
  return = g_object_new(GRAPH_WIDGET_TYPE, NULL);
}

Graph* get_graph(GraphWidget* gw) {
  return gw->graph;
}
